# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google_internal_apis/input.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from google.type import date_pb2 as google_dot_type_dot_date__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n google_internal_apis/input.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x16google/type/date.proto\"S\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x02(\t\x12\x0e\n\x06tag_id\x18\x02 \x02(\t\x12.\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"X\n\x06Tagged\x12\x0f\n\x07\x62ook_id\x18\x01 \x02(\t\x12\x0e\n\x06tag_id\x18\x02 \x02(\t\x12-\n\ttagged_at\x18\x03 \x02(\x0b\x32\x1a.google.protobuf.Timestamp\"A\n\x0cTagsResponse\x12\x12\n\x04tags\x18\x01 \x03(\x0b\x32\x04.Tag\x12\x1d\n\x0ctagged_items\x18\x02 \x03(\x0b\x32\x07.Tagged\"+\n\nTagRequest\x12\x1d\n\x0ctagged_items\x18\x01 \x03(\x0b\x32\x07.Tagged\"F\n\x17LibraryDocumentResponse\x12+\n\x11library_documents\x18\x01 \x02(\x0b\x32\x10.LibraryDocument\"\xd3\t\n\x0fLibraryDocument\x12\x0f\n\x07\x62ook_id\x18\x01 \x02(\t\x12#\n\x04\x62ook\x18\x02 \x02(\x0b\x32\x15.LibraryDocument.Book\x1a\x89\t\n\x04\x42ook\x12\r\n\x05title\x18\x01 \x02(\t\x12\x0f\n\x07\x61uthors\x18\x02 \x03(\t\x12\x11\n\tpublisher\x18\x03 \x02(\t\x12)\n\x0epublished_date\x18\x04 \x02(\x0b\x32\x11.google.type.Date\x12\x13\n\x0b\x64\x65scription\x18\x05 \x02(\t\x12\x12\n\npage_count\x18\x06 \x02(\x05\x12\x0f\n\x07version\x18\x07 \x02(\t\x12\x36\n\x0c\x63over_images\x18\x08 \x03(\x0b\x32 .LibraryDocument.Book.CoverImage\x12\x10\n\x08language\x18\t \x02(\t\x12\x0b\n\x03url\x18\n \x02(\t\x12\x10\n\x08unknown1\x18\x0b \x02(\x05\x12\x11\n\tunknown2s\x18\x0c \x03(\x05\x12\x10\n\x08unknown3\x18\r \x02(\x05\x12\x10\n\x08unknown4\x18\x0e \x02(\x05\x12,\n\x06series\x18\x0f \x02(\x0b\x32\x1c.LibraryDocument.Book.Series\x12\x11\n\tunknown5s\x18\x10 \x03(\x05\x12.\n\x08unknown6\x18\x11 \x01(\x0b\x32\x1c.google.protobuf.StringValue\x12\'\n\x04geos\x18\x12 \x03(\x0b\x32\x19.LibraryDocument.Book.Geo\x12.\n\x08unknown7\x18\x13 \x02(\x0b\x32\x1c.google.protobuf.StringValue\x12,\n\x06rating\x18\x14 \x02(\x0b\x32\x1c.LibraryDocument.Book.Rating\x12/\n\x06genres\x18\x15 \x03(\x0b\x32\x1f.LibraryDocument.Book.BookGenre\x1a\x37\n\tBookGenre\x12*\n\x05genre\x18\x01 \x02(\x0b\x32\x1b.LibraryDocument.Book.Genre\x1a\"\n\x03Geo\x12\x0b\n\x03geo\x18\x01 \x02(\t\x12\x0e\n\x06geo_id\x18\x02 \x02(\t\x1a(\n\x05Genre\x12\r\n\x05genre\x18\x01 \x02(\t\x12\x10\n\x08subgenre\x18\x02 \x01(\t\x1a\x33\n\x06Rating\x12\x0e\n\x06rating\x18\x01 \x02(\x01\x12\x19\n\x11number_of_reviews\x18\x02 \x02(\t\x1am\n\x06Series\x12\x17\n\x0fthis_book_title\x18\x01 \x02(\t\x12\x32\n\x06series\x18\x02 \x03(\x0b\x32\".LibraryDocument.Book.SingleSeries\x12\x16\n\x0e\x62ook_in_series\x18\x03 \x02(\t\x1a\x9c\x01\n\x0cSingleSeries\x12\x11\n\tseries_id\x18\x01 \x02(\t\x12\x10\n\x08unknown1\x18\x02 \x02(\x05\x12\x10\n\x08unknown2\x18\x03 \x02(\x05\x12.\n\x08unknown3\x18\x04 \x02(\x0b\x32\x1c.google.protobuf.StringValue\x12\x10\n\x08unknown4\x18\x05 \x02(\x05\x12\x13\n\x0bseries_name\x18\x06 \x02(\t\x1aV\n\nCoverImage\x12\x0b\n\x03url\x18\x01 \x02(\t\x12\r\n\x05scale\x18\x02 \x02(\x05\x12\r\n\x05\x63olor\x18\x03 \x02(\t\x12\r\n\x05width\x18\x04 \x02(\x05\x12\x0e\n\x06height\x18\x05 \x02(\x05')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google_internal_apis.input_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_TAG']._serialized_start=125
  _globals['_TAG']._serialized_end=208
  _globals['_TAGGED']._serialized_start=210
  _globals['_TAGGED']._serialized_end=298
  _globals['_TAGSRESPONSE']._serialized_start=300
  _globals['_TAGSRESPONSE']._serialized_end=365
  _globals['_TAGREQUEST']._serialized_start=367
  _globals['_TAGREQUEST']._serialized_end=410
  _globals['_LIBRARYDOCUMENTRESPONSE']._serialized_start=412
  _globals['_LIBRARYDOCUMENTRESPONSE']._serialized_end=482
  _globals['_LIBRARYDOCUMENT']._serialized_start=485
  _globals['_LIBRARYDOCUMENT']._serialized_end=1720
  _globals['_LIBRARYDOCUMENT_BOOK']._serialized_start=559
  _globals['_LIBRARYDOCUMENT_BOOK']._serialized_end=1720
  _globals['_LIBRARYDOCUMENT_BOOK_BOOKGENRE']._serialized_start=1176
  _globals['_LIBRARYDOCUMENT_BOOK_BOOKGENRE']._serialized_end=1231
  _globals['_LIBRARYDOCUMENT_BOOK_GEO']._serialized_start=1233
  _globals['_LIBRARYDOCUMENT_BOOK_GEO']._serialized_end=1267
  _globals['_LIBRARYDOCUMENT_BOOK_GENRE']._serialized_start=1269
  _globals['_LIBRARYDOCUMENT_BOOK_GENRE']._serialized_end=1309
  _globals['_LIBRARYDOCUMENT_BOOK_RATING']._serialized_start=1311
  _globals['_LIBRARYDOCUMENT_BOOK_RATING']._serialized_end=1362
  _globals['_LIBRARYDOCUMENT_BOOK_SERIES']._serialized_start=1364
  _globals['_LIBRARYDOCUMENT_BOOK_SERIES']._serialized_end=1473
  _globals['_LIBRARYDOCUMENT_BOOK_SINGLESERIES']._serialized_start=1476
  _globals['_LIBRARYDOCUMENT_BOOK_SINGLESERIES']._serialized_end=1632
  _globals['_LIBRARYDOCUMENT_BOOK_COVERIMAGE']._serialized_start=1634
  _globals['_LIBRARYDOCUMENT_BOOK_COVERIMAGE']._serialized_end=1720
# @@protoc_insertion_point(module_scope)



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.type import date_pb2 as _date_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Tag(_message.Message):
    __slots__ = ["name", "tag_id", "created_at"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAG_ID_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    name: str
    tag_id: str
    created_at: _timestamp_pb2.Timestamp
    def __init__(self, name: _Optional[str] = ..., tag_id: _Optional[str] = ..., created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class Tagged(_message.Message):
    __slots__ = ["book_id", "tag_id", "tagged_at"]
    BOOK_ID_FIELD_NUMBER: _ClassVar[int]
    TAG_ID_FIELD_NUMBER: _ClassVar[int]
    TAGGED_AT_FIELD_NUMBER: _ClassVar[int]
    book_id: str
    tag_id: str
    tagged_at: _timestamp_pb2.Timestamp
    def __init__(self, book_id: _Optional[str] = ..., tag_id: _Optional[str] = ..., tagged_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class TagsResponse(_message.Message):
    __slots__ = ["tags", "tagged_items"]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    TAGGED_ITEMS_FIELD_NUMBER: _ClassVar[int]
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    tagged_items: _containers.RepeatedCompositeFieldContainer[Tagged]
    def __init__(self, tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ..., tagged_items: _Optional[_Iterable[_Union[Tagged, _Mapping]]] = ...) -> None: ...

class TagRequest(_message.Message):
    __slots__ = ["tagged_items"]
    TAGGED_ITEMS_FIELD_NUMBER: _ClassVar[int]
    tagged_items: _containers.RepeatedCompositeFieldContainer[Tagged]
    def __init__(self, tagged_items: _Optional[_Iterable[_Union[Tagged, _Mapping]]] = ...) -> None: ...

class LibraryDocumentResponse(_message.Message):
    __slots__ = ["library_documents"]
    LIBRARY_DOCUMENTS_FIELD_NUMBER: _ClassVar[int]
    library_documents: LibraryDocument
    def __init__(self, library_documents: _Optional[_Union[LibraryDocument, _Mapping]] = ...) -> None: ...

class LibraryDocument(_message.Message):
    __slots__ = ["book_id", "book"]
    class Book(_message.Message):
        __slots__ = ["title", "authors", "publisher", "published_date", "description", "page_count", "version", "cover_images", "language", "url", "unknown1", "unknown2s", "unknown3", "unknown4", "series", "unknown5s", "unknown6", "geos", "unknown7", "rating", "genres"]
        class BookGenre(_message.Message):
            __slots__ = ["genre"]
            GENRE_FIELD_NUMBER: _ClassVar[int]
            genre: LibraryDocument.Book.Genre
            def __init__(self, genre: _Optional[_Union[LibraryDocument.Book.Genre, _Mapping]] = ...) -> None: ...
        class Geo(_message.Message):
            __slots__ = ["geo", "geo_id"]
            GEO_FIELD_NUMBER: _ClassVar[int]
            GEO_ID_FIELD_NUMBER: _ClassVar[int]
            geo: str
            geo_id: str
            def __init__(self, geo: _Optional[str] = ..., geo_id: _Optional[str] = ...) -> None: ...
        class Genre(_message.Message):
            __slots__ = ["genre", "subgenre"]
            GENRE_FIELD_NUMBER: _ClassVar[int]
            SUBGENRE_FIELD_NUMBER: _ClassVar[int]
            genre: str
            subgenre: str
            def __init__(self, genre: _Optional[str] = ..., subgenre: _Optional[str] = ...) -> None: ...
        class Rating(_message.Message):
            __slots__ = ["rating", "number_of_reviews"]
            RATING_FIELD_NUMBER: _ClassVar[int]
            NUMBER_OF_REVIEWS_FIELD_NUMBER: _ClassVar[int]
            rating: float
            number_of_reviews: str
            def __init__(self, rating: _Optional[float] = ..., number_of_reviews: _Optional[str] = ...) -> None: ...
        class Series(_message.Message):
            __slots__ = ["this_book_title", "series", "book_in_series"]
            THIS_BOOK_TITLE_FIELD_NUMBER: _ClassVar[int]
            SERIES_FIELD_NUMBER: _ClassVar[int]
            BOOK_IN_SERIES_FIELD_NUMBER: _ClassVar[int]
            this_book_title: str
            series: _containers.RepeatedCompositeFieldContainer[LibraryDocument.Book.SingleSeries]
            book_in_series: str
            def __init__(self, this_book_title: _Optional[str] = ..., series: _Optional[_Iterable[_Union[LibraryDocument.Book.SingleSeries, _Mapping]]] = ..., book_in_series: _Optional[str] = ...) -> None: ...
        class SingleSeries(_message.Message):
            __slots__ = ["series_id", "unknown1", "unknown2", "unknown3", "unknown4", "series_name"]
            SERIES_ID_FIELD_NUMBER: _ClassVar[int]
            UNKNOWN1_FIELD_NUMBER: _ClassVar[int]
            UNKNOWN2_FIELD_NUMBER: _ClassVar[int]
            UNKNOWN3_FIELD_NUMBER: _ClassVar[int]
            UNKNOWN4_FIELD_NUMBER: _ClassVar[int]
            SERIES_NAME_FIELD_NUMBER: _ClassVar[int]
            series_id: str
            unknown1: int
            unknown2: int
            unknown3: _wrappers_pb2.StringValue
            unknown4: int
            series_name: str
            def __init__(self, series_id: _Optional[str] = ..., unknown1: _Optional[int] = ..., unknown2: _Optional[int] = ..., unknown3: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ..., unknown4: _Optional[int] = ..., series_name: _Optional[str] = ...) -> None: ...
        class CoverImage(_message.Message):
            __slots__ = ["url", "scale", "color", "width", "height"]
            URL_FIELD_NUMBER: _ClassVar[int]
            SCALE_FIELD_NUMBER: _ClassVar[int]
            COLOR_FIELD_NUMBER: _ClassVar[int]
            WIDTH_FIELD_NUMBER: _ClassVar[int]
            HEIGHT_FIELD_NUMBER: _ClassVar[int]
            url: str
            scale: int
            color: str
            width: int
            height: int
            def __init__(self, url: _Optional[str] = ..., scale: _Optional[int] = ..., color: _Optional[str] = ..., width: _Optional[int] = ..., height: _Optional[int] = ...) -> None: ...
        TITLE_FIELD_NUMBER: _ClassVar[int]
        AUTHORS_FIELD_NUMBER: _ClassVar[int]
        PUBLISHER_FIELD_NUMBER: _ClassVar[int]
        PUBLISHED_DATE_FIELD_NUMBER: _ClassVar[int]
        DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
        PAGE_COUNT_FIELD_NUMBER: _ClassVar[int]
        VERSION_FIELD_NUMBER: _ClassVar[int]
        COVER_IMAGES_FIELD_NUMBER: _ClassVar[int]
        LANGUAGE_FIELD_NUMBER: _ClassVar[int]
        URL_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN1_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN2S_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN3_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN4_FIELD_NUMBER: _ClassVar[int]
        SERIES_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN5S_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN6_FIELD_NUMBER: _ClassVar[int]
        GEOS_FIELD_NUMBER: _ClassVar[int]
        UNKNOWN7_FIELD_NUMBER: _ClassVar[int]
        RATING_FIELD_NUMBER: _ClassVar[int]
        GENRES_FIELD_NUMBER: _ClassVar[int]
        title: str
        authors: _containers.RepeatedScalarFieldContainer[str]
        publisher: str
        published_date: _date_pb2.Date
        description: str
        page_count: int
        version: str
        cover_images: _containers.RepeatedCompositeFieldContainer[LibraryDocument.Book.CoverImage]
        language: str
        url: str
        unknown1: int
        unknown2s: _containers.RepeatedScalarFieldContainer[int]
        unknown3: int
        unknown4: int
        series: LibraryDocument.Book.Series
        unknown5s: _containers.RepeatedScalarFieldContainer[int]
        unknown6: _wrappers_pb2.StringValue
        geos: _containers.RepeatedCompositeFieldContainer[LibraryDocument.Book.Geo]
        unknown7: _wrappers_pb2.StringValue
        rating: LibraryDocument.Book.Rating
        genres: _containers.RepeatedCompositeFieldContainer[LibraryDocument.Book.BookGenre]
        def __init__(self, title: _Optional[str] = ..., authors: _Optional[_Iterable[str]] = ..., publisher: _Optional[str] = ..., published_date: _Optional[_Union[_date_pb2.Date, _Mapping]] = ..., description: _Optional[str] = ..., page_count: _Optional[int] = ..., version: _Optional[str] = ..., cover_images: _Optional[_Iterable[_Union[LibraryDocument.Book.CoverImage, _Mapping]]] = ..., language: _Optional[str] = ..., url: _Optional[str] = ..., unknown1: _Optional[int] = ..., unknown2s: _Optional[_Iterable[int]] = ..., unknown3: _Optional[int] = ..., unknown4: _Optional[int] = ..., series: _Optional[_Union[LibraryDocument.Book.Series, _Mapping]] = ..., unknown5s: _Optional[_Iterable[int]] = ..., unknown6: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ..., geos: _Optional[_Iterable[_Union[LibraryDocument.Book.Geo, _Mapping]]] = ..., unknown7: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ..., rating: _Optional[_Union[LibraryDocument.Book.Rating, _Mapping]] = ..., genres: _Optional[_Iterable[_Union[LibraryDocument.Book.BookGenre, _Mapping]]] = ...) -> None: ...
    BOOK_ID_FIELD_NUMBER: _ClassVar[int]
    BOOK_FIELD_NUMBER: _ClassVar[int]
    book_id: str
    book: LibraryDocument.Book
    def __init__(self, book_id: _Optional[str] = ..., book: _Optional[_Union[LibraryDocument.Book, _Mapping]] = ...) -> None: ...



from email.message import Message

import ghunt.globals as gb
import httpx
from ghunt.knowledge.keys import keys
from ghunt.objects.apis import GAPI
from ghunt.objects.base import GHuntCreds

keys.update(
    {
        "play": {
            "key": "AIzaSyCWq1--9JnN9QM7k57Rc_qmt9c0OVy0rME",
            "origin": "https://play.google.com",
        },
        "waa": {
            "key": "AIzaSyBGb5fGAyC-pRcRU6MUHb__b_vKha71HRE",
            "origin": "https://www.googleapis.com",  # TODO: confirm this value
        },
    }
)


class RpcService(GAPI):
    def __init__(
        self,
        creds: GHuntCreds,
        as_client: httpx.AsyncClient,
        headers: dict[str, str] = {},
    ):
        super().__init__()

        if not headers:
            headers = gb.config.headers

        base_headers = {
            "Content-Type": "application/json+protobuf",
            "X-User-Agent": "grpc-web-javascript/0.1",
        }

        headers = {**headers, **base_headers}

        self.scheme = "https"
        self.as_client = as_client

        self.authentication_mode = (
            "sapisidhash"  # sapisidhash, cookies_only, oauth or None
        )
        self.require_key = "play"  # key name, or None

        self._load_api(creds, headers)

    async def _call_rpc(self, method, data=None):
        self._load_endpoint(method)

        message = Message()
        for k, v in self.loaded_endpoints[method].headers.items():
            message.add_header(k, v)

        res = await self._query(
            self.as_client,
            "POST",
            method,
            f"/$rpc/{self.service}/{method}",
            {"$httpHeaders": message.as_string()},
            data,
            "data" if isinstance(data, str) else "json",
        )
        try:
            data = res.json()
        except ValueError:
            data = res.text
        if not res.is_success:
            raise Exception(res, data)
        return data



from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DummyMessage(_message.Message):
    __slots__ = ["dummy_field"]
    DUMMY_FIELD_FIELD_NUMBER: _ClassVar[int]
    dummy_field: _wrappers_pb2.StringValue
    def __init__(self, dummy_field: _Optional[_Union[_wrappers_pb2.StringValue, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google_internal_apis/dummy.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n google_internal_apis/dummy.proto\x1a\x1egoogle/protobuf/wrappers.proto\"A\n\x0c\x44ummyMessage\x12\x31\n\x0b\x64ummy_field\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.StringValue')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google_internal_apis.dummy_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_DUMMYMESSAGE']._serialized_start=68
  _globals['_DUMMYMESSAGE']._serialized_end=133
# @@protoc_insertion_point(module_scope)



from typing import TypeVar

import httpx
from ghunt.helpers import auth

from .endpoints import *  # noqa
from .ghunter import RpcService

__version__ = "0.3.1"
T = TypeVar("T", bound="RpcService")


async def get_client(t: type[T]) -> T:
    client = httpx.AsyncClient()
    creds = await auth.load_and_auth(client)
    return t(creds, client)



import warnings
from datetime import date, datetime
from pprint import pformat

from google.protobuf.timestamp_pb2 import Timestamp
from google.protobuf.wrappers_pb2 import StringValue
from google.type.date_pb2 import Date

__all__ = ["from_datetime", "dump", "parse"]


def from_datetime(dt: datetime) -> Timestamp:
    """
    Convert a datetime object to a protobuf Timestamp object
    """
    m = Timestamp()
    m.FromDatetime(dt)
    return m


def dump_repeated(message, field):
    if field.type == field.TYPE_MESSAGE:
        return [dump(item) for item in getattr(message, field.name)]
    return getattr(message, field.name)


def dump(message):
    """
    Convert a protobuf message to a recursive list of values
    """
    if isinstance(message, Timestamp):
        return str(message.ToMilliseconds())
    if isinstance(message, StringValue):
        return message.value if message.value else None
    if isinstance(message, Date):
        return date(year=message.year, month=message.month, day=message.day).isoformat()
    fields = message.DESCRIPTOR.fields
    arrays = []
    for field in fields:
        match field.label:
            case field.LABEL_REPEATED:
                arrays.append(dump_repeated(message, field))
            case field.LABEL_REQUIRED:
                if field.type == field.TYPE_MESSAGE:
                    arrays.append(dump(getattr(message, field.name)))
                else:
                    arrays.append(getattr(message, field.name))
            case field.LABEL_OPTIONAL:
                if field.type == field.TYPE_MESSAGE:
                    arrays.append(dump(getattr(message, field.name)))
                else:
                    arrays.append(getattr(message, field.name))
            case _:
                raise ValueError("Unknown label")
    return arrays


def repeated(message, field, value):
    if field.type == field.TYPE_MESSAGE:
        for array in value:
            parse(array, getattr(message, field.name).add())
    else:
        getattr(message, field.name).extend(value)


def required(message, field, value):
    if field.type == field.TYPE_MESSAGE:
        parse(value, getattr(message, field.name))
    else:
        setattr(message, field.name, value)


def optional(message, field, value):
    if field.type == field.TYPE_MESSAGE:
        parse(value, getattr(message, field.name))
    else:
        setattr(message, field.name, value)


def parse(arrays, message):
    """
    Convert a recursive list of values to a protobuf message
    """
    fields = message.DESCRIPTOR.fields
    if isinstance(message, Timestamp):
        message.FromMilliseconds(int(arrays))
        return message
    if isinstance(message, StringValue):
        if arrays:
            message.value = arrays
        return message
    if isinstance(message, Date):
        dt = date.fromisoformat(arrays)
        message.year = dt.year
        message.month = dt.month
        message.day = dt.day
        return message

    for field in fields:
        parse_field(message, field, arrays)
    remaining = arrays[fields[-1].number :]
    if remaining:
        warnings.warn("Extra fields: " + pformat(remaining))
    return message


def parse_field(message, field, arrays):
    match field.label:
        case field.LABEL_REPEATED:
            value = arrays[field.number - 1]
            repeated(message, field, value)
        case field.LABEL_REQUIRED:
            value = arrays[field.number - 1]
            required(message, field, value)
        case field.LABEL_OPTIONAL:
            if len(arrays) >= field.number:
                value = arrays[field.number - 1]
                optional(message, field, value)
        case _:
            raise ValueError("Unknown label")



from .ghunter import RpcService


class WhApiServiceRpc(RpcService):
    hostname = "playatoms-pa.clients6.google.com"
    service = "google.internal.play.atoms.api.v1.WhApiService"

    async def get_app_splits(self, data=None):
        return await self._call_rpc("GetAppSplits", data=data)

    async def update_user_prefs(self, data=None):
        return await self._call_rpc("UpdateUserPrefs", data=data)

    async def get_domain_filter(self, data=None):
        return await self._call_rpc("GetDomainFilter", data=data)

    async def get_intent_filter(self, data=None):
        return await self._call_rpc("GetIntentFilter", data=data)


class InternalPeopleServiceRpc(RpcService):
    hostname = "people-pa.clients6.google.com"
    service = "google.internal.people.v2.InternalPeopleService"

    async def get_people(
        self,
        data='[["me"],[[["person.name","person.email","person.photo"]],null,[5,1,7]]]',
    ):
        return await self._call_rpc("GetPeople", data=data)

    async def get_person_photo_encoded(self, data=None):
        return await self._call_rpc("GetPersonPhotoEncoded", data=data)

    async def update_person_photo(self, data=None):
        return await self._call_rpc("UpdatePersonPhoto", data=data)


class InternalPeopleMinimalServiceRpc(RpcService):
    hostname = "people-pa.clients6.google.com"
    service = "google.internal.people.v2.minimal.InternalPeopleMinimalService"

    async def list_people_by_known_id(self, data=None):
        return await self._call_rpc("ListPeopleByKnownId", data=data)


class AudiobookServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.audiobook.v1.AudiobookService"

    async def get_audio_events(
        self, data='["AQAAAEAszz6PAM","8af46ef0d127346baff59518",[8],4976830,6776830]'
    ):
        return await self._call_rpc("GetAudioEvents", data=data)

    async def get_audiobook_resource(self, data=None):
        return await self._call_rpc("GetAudiobookResource", data=data)

    async def get_audiobook_supplement(self, data=None):
        return await self._call_rpc("GetAudiobookSupplement", data=data)


class CloudLoadingOnePlatformServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = (
        "google.internal.play.books.cloudloading.v1.CloudLoadingOnePlatformService"
    )

    async def delete(self, data=None):
        return await self._call_rpc("Delete", data=data)

    async def insert(self, data=None):
        return await self._call_rpc("Insert", data=data)


class EnterpriseServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.enterprise.v1.EnterpriseService"

    async def add_classroom_books_attachment(self, data=None):
        return await self._call_rpc("AddClassroomBooksAttachment", data=data)

    async def add_license_owners(self, data=None):
        return await self._call_rpc("AddLicenseOwners", data=data)

    async def add_license_redeemers(self, data=None):
        return await self._call_rpc("AddLicenseRedeemers", data=data)

    async def claim_license(self, data=None):
        return await self._call_rpc("ClaimLicense", data=data)

    async def get_license_history(self, data=None):
        return await self._call_rpc("GetLicenseHistory", data=data)

    async def get_licenses(self, data="[null,null,null,1]"):
        return await self._call_rpc("GetLicenses", data=data)

    async def grant_free_book_licenses(self, data=None):
        return await self._call_rpc("GrantFreeBookLicenses", data=data)

    async def list_free_books_for_license(self, data=None):
        return await self._call_rpc("ListFreeBooksForLicense", data=data)

    async def remove_license_owners(self, data=None):
        return await self._call_rpc("RemoveLicenseOwners", data=data)

    async def remove_license_redeemers(self, data=None):
        return await self._call_rpc("RemoveLicenseRedeemers", data=data)


class FamilyOnePlatformServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.family.v1.FamilyOnePlatformService"

    async def share(self, data=None):
        return await self._call_rpc("Share", data=data)

    async def unshare(self, data=None):
        return await self._call_rpc("Unshare", data=data)


class VolumeAnnotationServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.layers.v1.VolumeAnnotationService"

    async def get_dictionary_definition(
        self, data='[null," unwrapping",[null,null,"en"]]'
    ):
        return await self._call_rpc("GetDictionaryDefinition", data=data)


class LibraryServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.library.v1.LibraryService"

    async def add_annotation(
        self,
        data='[["AQAAAEAszz6PAM","AE67D741-9B6D-4C7D-B170-312AF12BE063",[],null,null,[["40102"]],"0.10.0"]]',
    ):
        return await self._call_rpc("AddAnnotation", data=data)

    async def add_library_document(self, data=None):
        return await self._call_rpc("AddLibraryDocument", data=data)

    async def add_tags(
        self, data='[[["5QuwNwAAAEAJ","a0c746137-5723-4740-89a8-4ef7311aad16"]]]'
    ):
        return await self._call_rpc("AddTags", data=data)

    async def create_custom_tag(self, data=None):
        return await self._call_rpc("CreateCustomTag", data=data)

    async def delete_annotation(self, data=None):
        return await self._call_rpc("DeleteAnnotation", data=data)

    async def delete_custom_tag(self, data=None):
        return await self._call_rpc("DeleteCustomTag", data=data)

    async def delete_library_document(self, data=None):
        return await self._call_rpc("DeleteLibraryDocument", data=data)

    async def expunge_library_documents(self, data=None):
        return await self._call_rpc("ExpungeLibraryDocuments", data=data)

    async def get_library_document(self, data='[[], "hdYTCAAAQBAJ"]'):
        return await self._call_rpc("GetLibraryDocument", data=data)

    async def get_volume_overviews(self, data=None):
        return await self._call_rpc("GetVolumeOverviews", data=data)

    async def hide_library_documents(self, data=None):
        return await self._call_rpc("HideLibraryDocuments", data=data)

    async def list_annotations(self, data=None):
        return await self._call_rpc("ListAnnotations", data=data)

    async def list_tags(self, data=None):
        return await self._call_rpc("ListTags", data=data)

    async def lookup_copy_limit(self, data=None):
        return await self._call_rpc("LookupCopyLimit", data=data)

    async def remove_tags(self, data='[[["hdYTCAAAQBAJ","Completed",1737103382078]]]'):
        return await self._call_rpc("RemoveTags", data=data)

    async def sync_document_position(
        self,
        data='["84XKNwAAAEAJ",["384DC38E-1A56-4F48-8AE1-1C3F446F41E5",null,"1737093647557",[null,null,[2]],null,[["GBS.PA68.w.0.0.0.2"],null,0.9066666666666666]]]',
    ):
        return await self._call_rpc("SyncDocumentPosition", data=data)

    async def sync_extended_library(self, data=None):
        return await self._call_rpc("SyncExtendedLibrary", data=data)

    async def sync_user_library(self, data=None):
        return await self._call_rpc("SyncUserLibrary", data=data)

    async def unhide_library_documents(self, data=None):
        return await self._call_rpc("UnhideLibraryDocuments", data=data)

    async def update_copy_limit(self, data=None):
        return await self._call_rpc("UpdateCopyLimit", data=data)

    async def update_custom_tag(
        self, data='["a0c746137-5723-4740-89a8-4ef7311aad16","test 1"]'
    ):
        return await self._call_rpc("UpdateCustomTag", data=data)


class SeriesOnePlatformServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.series.v1.SeriesOnePlatformService"

    async def fetch(
        self,
        data='[[["u4P3GgAAABBNmM"],["PL4pGwAAABAUpM"],["bW8uGwAAABBCdM"],["i1YuGwAAABCkTM"],["crbMGgAAABC_rM"]]]',
    ):
        return await self._call_rpc("Fetch", data=data)

    async def fetch_members(self, data='[["u4P3GgAAABBNmM"],[100]]'):
        return await self._call_rpc("FetchMembers", data=data)


class SettingsOnePlatformServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.settings.v1.SettingsOnePlatformService"

    async def get_user_settings(self, data=None):
        return await self._call_rpc("GetUserSettings", data=data)

    async def update_user_settings(self, data=None):
        return await self._call_rpc("UpdateUserSettings", data=data)


class UserAnnotationServiceRpc(RpcService):
    hostname = "playbooks-pa.clients6.google.com"
    service = "google.internal.play.books.userannotations.v1.UserAnnotationService"

    async def create_annotations(self, data=None):
        return await self._call_rpc("CreateAnnotations", data=data)

    async def create_collection(self, data=None):
        return await self._call_rpc("CreateCollection", data=data)

    async def delete_annotations(self, data=None):
        return await self._call_rpc("DeleteAnnotations", data=data)

    async def delete_collection(self, data=None):
        return await self._call_rpc("DeleteCollection", data=data)

    async def list_annotations(
        self, data='[[["84XKNwAAAEAJ",null,"full-1.0.0"]],[1,2],null,1,1]'
    ):
        return await self._call_rpc("ListAnnotations", data=data)

    async def list_shared_annotations(
        self, data='[["84XKNwAAAEAJ",null,"full-1.0.0"],1,null,null,1]'
    ):
        return await self._call_rpc("ListSharedAnnotations", data=data)

    async def trigger_export(self, data=None):
        return await self._call_rpc("TriggerExport", data=data)

    async def update_annotations(self, data=None):
        return await self._call_rpc("UpdateAnnotations", data=data)

    async def update_collection(self, data=None):
        return await self._call_rpc("UpdateCollection", data=data)

    async def update_subscriptions(self, data=None):
        return await self._call_rpc("UpdateSubscriptions", data=data)


class PlayGatewayBooksServiceRpc(RpcService):
    hostname = "playgateway-pa.clients6.google.com"
    service = "play.gateway.adapter.books.v1.PlayGatewayBooksService"

    async def get_in_app_stream(
        self,
        data='[[4,null,["84XKNwAAAEAJ",1]],null,null,null,null,[null,null,null,null,null,null,[3]],null,null,[null,[2],[1]]]',
    ):
        return await self._call_rpc("GetInAppStream", data=data)

    async def update_item_wishlist_state(self, data='[null,[\\"ID07FerN_h0C\\",9],1]'):
        return await self._call_rpc("UpdateItemWishlistState", data=data)


class MakerSuiteServiceRpc(RpcService):
    hostname = "alkalimakersuite-pa.clients6.google.com"
    service = "google.internal.alkali.applications.makersuite.v1.MakerSuiteService"

    async def generate_content(self, data=None):
        return await self._call_rpc("GenerateContent", data=data)


class WaaRpc(RpcService):
    hostname = "waa-pa.clients6.google.com"
    service = "google.internal.waa.v1.Waa"

    async def create(self, data=None):
        return await self._call_rpc("Create", data=data)

    async def ping(self, data=None):
        return await self._call_rpc("Ping", data=data)
